package cn.ibizlab.core.analysis.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.analysis.domain.DAMetric;
import cn.ibizlab.core.analysis.filter.DAMetricSearchContext;
import cn.ibizlab.core.analysis.service.IDAMetricService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.analysis.mapper.DAMetricMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[指标] 服务对象接口实现
 */
@Slf4j
@Service("DAMetricServiceImpl")
public class DAMetricServiceImpl extends ServiceImpl<DAMetricMapper, DAMetric> implements IDAMetricService {

    @Autowired
    @Lazy
    protected cn.ibizlab.core.analysis.service.IDABuildService dabuildService;
    @Autowired
    @Lazy
    IDAMetricService proxyService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(DAMetric et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getMetricId()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<DAMetric> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(DAMetric et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("da_metricid", et.getMetricId()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getMetricId()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<DAMetric> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public DAMetric get(String key) {
        DAMetric et = getById(key);
        if(et == null){
            et = new DAMetric();
            et.setMetricId(key);
        }
        else {
        }
        return et;
    }

    @Override
    public DAMetric getDraft(DAMetric et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(DAMetric et) {
        return (!ObjectUtils.isEmpty(et.getMetricId())) && (!Objects.isNull(this.getById(et.getMetricId())));
    }
    @Override
    @Transactional
    public boolean save(DAMetric et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(DAMetric et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? proxyService.update(et) : proxyService.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<DAMetric> list) {
        list.forEach(item->fillParentData(item));
        List<DAMetric> create = new ArrayList<>();
        List<DAMetric> update = new ArrayList<>();
        for (DAMetric et : list) {
            if (ObjectUtils.isEmpty(et.getMetricId()) || ObjectUtils.isEmpty(getById(et.getMetricId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            proxyService.createBatch(create);
        }
        if (update.size() > 0) {
            proxyService.updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<DAMetric> list) {
        list.forEach(item->fillParentData(item));
        List<DAMetric> create = new ArrayList<>();
        List<DAMetric> update = new ArrayList<>();
        for (DAMetric et : list) {
            if (ObjectUtils.isEmpty(et.getMetricId()) || ObjectUtils.isEmpty(getById(et.getMetricId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            proxyService.createBatch(create);
        }
        if (update.size() > 0) {
            proxyService.updateBatch(update);
        }
    }

    @Override
    @Transactional
    public DAMetric syncRule(DAMetric et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean syncRuleBatch(List<DAMetric> etList) {
        for(DAMetric et : etList) {
            syncRule(et);
        }
        return true;
    }


	@Override
    public List<DAMetric> selectByBuildId(String build_id) {
        return baseMapper.selectByBuildId(build_id);
    }
    @Override
    public void removeByBuildId(String build_id) {
        this.remove(new QueryWrapper<DAMetric>().eq("buildid",build_id));
    }


    /**
     * 查询集合 数据集
     */
    @Override
    public Page<DAMetric> searchDefault(DAMetricSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<DAMetric> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<DAMetric>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(DAMetric et){
        //实体关系[DER1N_DA_METRIC_DA_BUILD_BUILDID]
        if(!ObjectUtils.isEmpty(et.getBuildId())){
            cn.ibizlab.core.analysis.domain.DABuild build=et.getBuild();
            if(ObjectUtils.isEmpty(build)){
                cn.ibizlab.core.analysis.domain.DABuild majorEntity=dabuildService.get(et.getBuildId());
                et.setBuild(majorEntity);
                build=majorEntity;
            }
            et.setBuildName(build.getBuildName());
            et.setModelId(build.getModelId());
            et.setModelName(build.getModelName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }





}



